<?php
define('ASG_APPLICATION_PATH', dirname(__FILE__));
define('ASG_LIBRARY_PATH', dirname(dirname(__FILE__)) . '/library');

// Define application environment
if (!defined('ASG_APPLICATION_ENV')) {
    if (getenv('ASG_APPLICATION_ENV')) {
        define('ASG_APPLICATION_ENV', getenv('ASG_APPLICATION_ENV'));
    } else if (getenv('REDIRECT_ASG_APPLICATION_ENV')) {
        define('ASG_APPLICATION_ENV', getenv('REDIRECT_ASG_APPLICATION_ENV'));
    }
    else {
        define('ASG_APPLICATION_ENV', 'production');
    }

}

abstract class Asg_Application
{
    static protected $_namespaces = array(
        'controller' => array(
            'namespace' => 'Asg_Controller',
            'folder'    => 'controllers',
        ),
        'model' => array(
            'namespace' => 'Asg_Model',
            'folder'    => 'models',
        ),
        'service' => array(
            'namespace' => 'Asg_Service',
            'folder'    => 'services',
        ),
        'form' => array(
            'namespace' => 'Asg_Form',
            'folder'    => 'forms',
        ),
    );

    static protected $_singletons = array();
    static protected $_config;

    static public function dispatch($controller='index', $action='index',
        $params = NULL)
    {
        if (empty($controller)) {
            $controller = 'index';
        } else {
            $controller = strtolower($controller);
        }
        if (empty($action)) {
            $action = 'index';
        } else {
            $action = strtolower($action);
        }

        $controllerClass = self::camelize($controller) . 'Controller';
        $controllerObject = self::factory($controllerClass, 'controller');

        $controllerObject->setParams((array) $params);

        $controllerObject->preDispatch($action);

        $actionName = self::camelize($action, FALSE);
        $method = $actionName . ('Action');
        $controllerObject->setViewScript($controller . '/' . $action . '.phtml');

        $result = $controllerObject->$method();

        $viewScript = $controllerObject->getViewScript();
        if (!empty($viewScript)) {
            /**
            * @var Ssc_WpModule_View_Engine
            */
            $view = $controllerObject->getView();

            if (($layout = $view->layout)
                && ($layoutScript = $controllerObject->getLayoutScript())
            ) {
                $layout->content = $view->render($viewScript);
                $layout->renderDirect($layoutScript);
            } else {
                $view->renderDirect($viewScript);
            }
        }

        $controllerObject->postDispatch($action);

        return $result;
    }

    static public function getModel($name)
    {
        return self::factory($name, 'model');
    }

    static public function getService($name)
    {
        return self::factory($name, 'service');
    }

    static public function getController($name)
    {
        return self::factory($name, 'controller');
    }

    static public function getForm($name)
    {
        return self::factory($name, 'form');
    }

    static public function factory($name, $type)
    {
        $info = self::$_namespaces[$type];
        $class = $info['namespace'] . '_' . $name;

        require_once ASG_APPLICATION_PATH . '/' . $info['folder'] . '/'
            . str_replace('_', '/', $name) . '.php';

        $singleton = method_exists($class, 'isSingleton')
            && call_user_func(array($class, 'isSingleton'));
        if ($singleton && isset(self::$_singletons[$class])) {
            return self::$_singletons[$class];
        }

        $result = new $class;

        if ($singleton) {
            self::$_singletons[$class] = $result;
        }

        return $result;
    }

    static public function camelize($string, $first=TRUE)
    {
        $string = ucwords(str_replace('-', ' ', $string));
        $string = str_replace(' ', '', $string);
        if (!$first) {
            $string = strtolower($string[0]) . substr($string, 1);
        }

        return $string;
    }

    static public function getConfig()
    {
        if (!self::$_config) {
            require_once ASG_LIBRARY_PATH . '/AsgZend/Config/Ini.php';
            self::$_config = new AsgZend_Config_Ini(ASG_APPLICATION_PATH . '/configs/application.ini', ASG_APPLICATION_ENV);
        }

        return self::$_config;
    }

    /**
    * Library factory
    */
    static public function loadLibrary($class)
    {
        require_once ASG_LIBRARY_PATH . '/' . str_replace('_', '/', $class) . '.php';
        $result = new $class;

        return $result;
    }
}