<?php
require_once ASG_APPLICATION_PATH . '/controllers/Abstract/Admin.php';

class Asg_Controller_AdminController
    extends Asg_Controller_Abstract_Admin
{
    public function indexAction()
    {
        /**
        * @var Asg_Model_AffiliateCore
        */
        $core = Asg_Application::getModel('AffiliateCore');

        if ($core->isActive()) {
            $this->_displayProgress();
            return;
        }

        // Unpack dialog data
        if (isset($_POST['dialog_data'])) {
            parse_str($_POST['dialog_data'], $dialogData);
            $_POST = array_merge($_POST, $dialogData);
            unset($_POST['dialog_data']);
        }

        /**
        * @var Asg_Form_Main
        */
        $form = Asg_Application::getForm('Admin')->loadDefaults();

        $action = '';

        try {
            if ($_POST) {
                if (isset($_POST['action'])) {
                    $action =  $_POST['action'];
                }

                if (!$form->isValid($_POST)) {
                    foreach ($form->getElements() as $element) {
                        foreach ($element->getMessages() as $message) {
                            $this->_addMessage(
                                $this->_view->escape("{$element->getLabel()}: {$message}"),
                                'error');
                        }
                    }

                    if ('add_selected' == $action) {
                        $this->_view->items = $_SESSION[Asg_WpPlugin::PREFIX . '_preview_items'];
                    }

                } else {
                    $formOptions = $form->exportValues();
                    $form->saveDefaults();

                    if ('preview' == $action) {
                        try {
                            $items = $core->preview($formOptions);
                            if (0 != count($items)) {
                                $this->_view->items = $items;
                                $this->_view->marketName = $core->getAmazonDomain();
                            } else {
                                $this->_addMessage('No products found for the search criteria.');
                            }
                            $_SESSION[Asg_WpPlugin::PREFIX . '_preview_items'] = $items;

                        } catch (Asg_Service_Affiliate_Exception $e) {
                            $this->_addMessage($this->_view->escape($e->getMessage()), 'error');
                        }
                    } else {
                        unset($_SESSION[Asg_WpPlugin::PREFIX]);

                        $this->_view->startOptions = $formOptions;
                        $this->_displayProgress();
                        return;
                    }
                }
            } else {
                $form->loadSearchCategories();
            }
        } catch (Asg_Service_Affiliate_Exception $e) {
            $this->_addMessage($this->_view->escape($e->getMessage()), 'error');
        }

        $this->_view->form = $form;
        $this->_view->action = $action;

        $this->_view->layout->announcement = Asg_Application::getModel('AnnouncementHtml')->get();
    }

    public function optionsAction()
    {
        $form = Asg_Application::getForm('Options')->loadDefaults();

        if (count($_POST)) {
            if (!$form->isValid($_POST)) {
                foreach ($form->getElements() as $element) {
                    foreach ($element->getMessages() as $message) {
                        $this->_addMessage(
                            $this->_view->escape("{$element->getLabel()}: {$message}"),
                            'error');
                    }
                }
            } else {
                try {
                    $form->saveDefaults();
                    $this->_addMessage('Settings saved');
                } catch (Asg_Form_Options_Exception $e) {
                    $message = $e->getMessage();
                    if (($element = $e->getElement())
                        && ($element = $form->getElement($element))
                    ) {
                        $message = "{$element->getLabel()}: {$message}";
                    }

                    $this->_addMessage(
                        $this->_view->escape($message),
                        'error');
                }
            }
        }

        $this->_view->form = $form;

        $this->_view->layout->announcement = Asg_Application::getModel('AnnouncementHtml')->get();
    }

    public function ajaxGetCategoriesAction()
    {
        $this->setViewScript(NULL);

        $result = array();
        $error = '';

        $credentials = Asg_Application::getModel('AffiliateCore')
            ->getCredentialFields();
        $requiredFields = $credentials;

        foreach ($requiredFields as $field) {
            if (!isset($_POST[$field]) || '' == $_POST[$field]) {
                $error = "Field '{$field}' is required";
                break;
            }
        }

        if ('' == $error) {
            try {
                $options = array();
                foreach ($credentials as $field) {
                    $options[$field] = $_POST[$field];
                }

                $parent = !isset($_POST['parent']) || '' == $_POST['parent']
                    ? NULL
                    : $_POST['parent'];

                $items = Asg_Application::getModel('AffiliateCore')
                    ->getCategories($parent, $options);

                foreach ($items as $id=>$label) {
                    $result['categories'][] = array(
                        'id' => $id,
                        'label' => $label,
                    );
                }
                // Preserve item order

            } catch (Asg_Service_Affiliate_Exception $e) {
                $error = $e->getMessage();
            }
        }

        header('Content-Type: application/json', TRUE);
        echo json_encode(array(
            'error' => $error,
            'result' => $result,
        ));
    }

    public function ajaxAddCategoryAction()
    {
        $this->setViewScript(NULL);

        $result = array();
        $error = '';

        $parent = null;;
        if (isset($_POST['parent'])) {
            $parent = $_POST['parent'];
        }

        if (!isset($_POST['newCat']) || ('' == $_POST['newCat'])) {
            $error = "Category name is required";

        } else {
            $newCat = $_POST['newCat'];

            $res = wp_insert_category(
                array(
                    'cat_name' => $newCat,
                    'category_parent' => $parent,
                ),
                TRUE
            );

           if (is_wp_error($res)) {
               $error = $res->get_error_message();
           } else {
               $category = get_category($res);
               if ($category) {
                   $result['id'] = $category->term_id;
                   $result['label'] = $category->name;
               } else {
                   $error = 'Unknown error';
               }
           }
        }

        header('Content-Type: application/json', TRUE);
        echo json_encode(array(
            'error' => $error,
            'result' => $result,
        ));
    }

    protected function _displayProgress()
    {
        $this->_view->updateTimeout = 3000;
        $this->_viewScript = 'progress/index.phtml';
    }
}