<?php
require_once ASG_APPLICATION_PATH . '/controllers/Abstract/Filter.php';

class Asg_Controller_AmazonDataController
    extends Asg_Controller_Abstract_Filter
{
    protected $_model;

    protected $_imageSizes = array(
        'small'     => 'SmallImage',
        'medium'    => 'MediumImage',
        'large'     => 'LargeImage',
        'tiny'      => 'TinyImage',
        'thumbnail' => 'ThumbnailImage',
        'swatch'    => 'SwatchImage',
    );

    public function preDispatch(&$action)
    {
        parent::preDispatch($action);

        $this->_model = Asg_Application::getModel('Amazon_Data')
            ->setData($this->getParam('data'))
            ->setPostId($this->getParam('object_id'));
    }

    public function indexAction()
    {
        $key = $this->getParam('meta_key');
        switch ($key) {
            case 'price':
                return $this->_model->extractPrice();
            case 'list-price':
                return $this->_model->extractListPrice();
            case 'product-url':
                return $this->_model->extractProductUrl();

            case 'image-url':
                return $this->_model->extractImageUrl();
            case 'image-data':
                return $this->_model->extractImageData();
        }

        if ('image-url-' == substr($key, 0, 10)) {
            $size = substr($key, 10);
            if (isset($this->_imageSizes[$size])) {
                return $this->_model->extractImageUrl($this->_imageSizes[$size]);
            }
        } else if ('image-data-' == substr($key, 0, 11)) {
            $size = substr($key, 11);
            if (isset($this->_imageSizes[$size])) {
                $result = $this->_model->extractImageData($this->_imageSizes[$size]);
                // Lowercase array keys
                return array(
                    'url' => $result['URL'],
                    'width' => $result['Width'],
                    'height' => $result['Height'],
                );
            }
        }

        return NULL;
    }

    public function dataAction()
    {
        $key = $this->getParam('meta_key');
        $path = explode('-', $key);
        array_shift($path);

        return $this->_model->extractElement($path);
    }
}