<?php
require_once 'AsgZend/Form.php';

require_once 'AsgPumka/Filter/KeywordList.php';
require_once 'AsgPumka/Filter/AsinList.php';
require_once 'AsgZend/Validate/StringLength.php';

class Asg_Form_Admin extends AsgZend_Form
{
    protected $_possiblyDisabledElements = array(
        'posts_per_week',
        'price_min',
        'price_max',
        'search_category',
        'max_items',
    );

    protected $_markets;

    protected $_searchCategoryFields = array();

    public function init()
    {
        /**
        * @var Asg_Model_AffiliateCore
        */
        $core = Asg_Application::getModel('AffiliateCore');
        $multiOptions = $core->getMultiOptions();
        $constraints = $core->getConstraints();

        $this->setDisableTranslator(TRUE);

        //
        // Market fields
        //

        $this->_markets = Asg_Application::getModel('Amazon_Market')->getList();
        foreach ($this->_markets as $marketCode=>$marketLabel) {
            $this->addElement('text', "{$marketCode}_affiliate_id", array(
                'filters' => array(
                    'StripNewlines',
                    'StringTrim',
                    'Null',
                ),
                'validators' => array(
                    array('Regex', TRUE,
                        array(
                            'pattern' => '~^[a-zA-Z0-9\-\.]+$~u',
                            'messages' => array(
                                'regexNotMatch' => "'%value%' contains invalid characters",
                            ),
                        ),
                    ),
                ),
                'label' => "{$marketLabel} Associate ID",
                '-help' => FALSE,
            ));
        }

        //
        // Credentials
        //

        $this->addElement('text', 'amazon_key', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
            ),
            'label' => 'Access Key ID'
        ));

        $this->addElement('password', 'amazon_secret_key', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
            ),
            'label' => 'Secret Access Key'
        ));

        // Scheduling options

        $this->addElement('radio', 'publish_type', array(
            'required' => TRUE,
            'multiOptions' => array(
                'schedule' => 'Schedule',
                'immediately' => 'Publish all posts imeddiately',
            ),
            'label' => 'Scheduling options:',
        ));

        $this->addElement('select', 'posts_per_week', array(
            'multiOptions' => $multiOptions['posts_per_week'],
            //'required' => TRUE,
            'label' => 'Schedule posts per week',
        ));

        $this->addElement('text', 'max_items', array(
            'required' => TRUE,
            'filters' => array(
                'Digits',
            ),
            'validators' => array(
                'Digits',
                array('Between', TRUE, array(
                    'min' => $constraints['max_items']['min'],
                    'max' => $constraints['max_items']['max'],
                )),
            ),
            'label' => 'Add up to ... posts total',
        ));

        // Product search

        $this->addElement('radio', 'keywords_type', array(
            'required' => TRUE,
            'multiOptions' => array(
                'keyword' => 'Search By Keywords',
                'asin' => 'Search By ASIN numbers',
            ),
        ));

        $this->addElement('textarea', 'search_keywords', array(
            'filters' => array(
                new AsgPumka_Filter_KeywordList,
            ),
            'label' => 'Keywords',
        ));

        $this->addElement('select', 'amazon_market', array(
            //'required' => TRUE,
            'filters' => array(
                'Null',
            ),
            'multiOptions' => $this->_markets,
            'label' => 'Amazon Market'
        ));

        $this->addElement('multiselect', 'search_category', array(
            'label' => 'Amazon Category',
            'registerInArrayValidator' => FALSE,
        ));

        $this->addElement('text', 'price_min', array(
            'filters' => array(
                'Digits',
                array('Null', array('string')),
            ),
            'validators' => array(
                'Digits',
                array('GreaterThan', TRUE, array(
                    'min' => 0,
                )),
            ),
            'label' => 'Product Min Price',
            '-help' => FALSE,
        ));

        $this->addElement('text', 'price_max', array(
            'filters' => array(
                'Digits',
                array('Null', array('string')),
            ),
            'validators' => array(
                'Digits',
                array('GreaterThan', TRUE, array(
                    'min' => 0,
                )),
            ),
            'label' => 'Product Max Price',
            '-help' => FALSE,
        ));

        // Schedule posts under this category

        $this->addElement('multiCheckbox', 'post_category', array(
            'label' => 'Wordpress Category',
        ));

        $elem = $this->getElement('post_category');
        foreach(get_categories(array('hide_empty'=>0)) as $category) {
            $elem->addMultiOption($category->term_id, $category->name);
        }
    }

    public function isValid($data)
    {
        $action = isset($data['action'])? $data['action'] : '';

        if ('add_selected' == $action) {
            $items = $_SESSION[Asg_WpPlugin::PREFIX . '_preview_items'];
            if (!is_array($items)) {
                throw new Exception('Preview items not found');
            }
            $multiOptions = array();
            foreach ($items as $key=>$item) {
                if ($item['published']) {
                    continue;
                }

                $mutliOptions[$key] = $item['title'];
            }

            $this->addElement('multiCheckbox', 'select_items', array(
                'label' => 'Add Selected',
                'required' => TRUE,
                'multiOptions' => $mutliOptions,
                'validators' => array(
                    array('NotEmpty', TRUE,
                        array(
                            'messages' => array(
                                'isEmpty' => "At least one item must be selected",
                            ),
                        ),
                    ),
                ),
                '-help' => FALSE,
            ));
        }

        foreach ($this->_possiblyDisabledElements as $elemName) {
            if (!isset($data[$elemName])) {
                $data[$elemName] = $this->getValue($elemName);
            }
        }

        $categories = isset($data['search_category'])
            ? array_values((array) $data['search_category'])
            : array();
        $count = count($categories);
        for ($i=$count-1; $i>=0; $i--) {
            if (empty($categories[$i])) {
                unset($categories[$i]);
            } else {
                break;
            }
        }
        $data['search_category'] = $categories;

        // Load search categories
        $this->setDefaults($data);
        $this->loadSearchCategories();

        foreach ($this->_searchCategoryFields as $key=>$elem) {
            $data[$elem->getName()] = $elem->getValue();
        }

        $asinMode = 'asin' == $this->getValue('keywords_type');

        $keywordsElem = $this->getElement('search_keywords');
        $keywordsElem->setRequired($asinMode)
            ->addFilter($asinMode
                ? new AsgPumka_Filter_AsinList
                : new AsgPumka_Filter_KeywordList);
        if (!$asinMode) {
            $keywordsElem->addValidator(new AsgZend_Validate_StringLength(array('max' => 2000)), TRUE);
        }

        // Require affiliate ID for current market
        $market = $this->getValue('amazon_market');
        if ('' != $market) {
            $name = "{$market}_affiliate_id";
            $elem = $this->getElement($name);
            if ($elem) {
                $elem->setRequired(TRUE);
            }
        }

        $result = parent::isValid($data);

        if (!$asinMode) {
            if ('' == $this->getValue('search_category')
                && '' == $this->getValue('search_keywords')
            ) {
                $this->getElement('search_keywords')->addError('Keywords are required if no category selected');
                $result = FALSE;
            }

            $priceMin = $this->getValue('price_min');
            $priceMax = $this->getValue('price_max');

            if (''  != $priceMax && $priceMax < $priceMin) {
                $this->getElement('price_max')->addError("Maximum price must be greater than or equal to the minimum price");
                $result = FALSE;
            }

           if ((($priceMin && '' != $priceMin) || ($priceMax && '' != $priceMax))
                && $this->getValue('amazon_market')
            ) {
                $market = $this->getValue('amazon_market');
                $category = $this->getValue('search_category');
                if (!$category) {
                    $category = 'all';
                }

                $exception = Asg_Application::getConfig()
                    ->amazonExceptions->priceFilter->$market->$category;
                if (!is_null($exception)) {
                    $this->getElement('search_category')->addError($exception);
                    $result = FALSE;
                }
            }
        }

        return $result;
    }

    public function loadDefaults()
    {
        /**
        * @var Asg_Model_Options
        */
        $options = Asg_Application::getModel('Options');

        /**
        * @var Asg_Model_AffiliateCore
        */
        $defaults = Asg_Application::getModel('AffiliateCore')->getDefaults();

        foreach ($this->getElements() as $key=>$elem) {
            if (FALSE === $elem->getAttrib('-persistent')
                || FALSE === $elem->getAttrib('-persistent-load')
            ) {
                continue;
            }

            $elem->setValue(
                $options->getValue($key,
                isset($defaults[$key])? $defaults[$key] : NULL));
        }

        return $this;
    }

    public function saveDefaults()
    {
        /**
        * @var Asg_Model_Options
        */
        $options = Asg_Application::getModel('Options');

        foreach ($this->getElements() as $key=>$elem) {
            if (FALSE === $elem->getAttrib('-persistent')
                || FALSE === $elem->getAttrib('-persistent-save')
            ) {
                continue;
            }

            $options->setValue($key, $elem->getValue());
        }

        return $this;
    }

    public function loadSearchCategories()
    {
        $core = Asg_Application::getModel('AffiliateCore');

        if (!$this->hasCredentials()) {
            return $this;
        }
        $credentials = $this->exportCredentials();

        $selected = (array)$this->getValue('search_category');

        $count = count($selected);
        $parent = NULL;
        for ($i=0; $i<=$count; $i++) {
            $categories = $core->getCategories($parent, $credentials);
            if (!$categories) {
                // No subcategories
                break;
            }

            $parent = $i < $count? $selected[$i] : NULL;

            $label = 'Amazon Category';
            if ($i > 0) {
                $label .= ' (level ' . ($i+1) . ')';
            }
            $name = "search_category_{$i}";
            $this->addElement('select', $name, array(
                'multiOptions' => $categories,
                'filters' => array(
                    'Null',
                ),
                'required' => $i < $count,
                'value' => $parent,
                'label' => $label,
                '-name' => "search_category[{$i}]",
                '-persistent' => FALSE,
                '-help' => FALSE,
            ));

            $this->_searchCategoryFields[$i] = $this->getElement($name);
        }

        return $this;
    }

    public function exportValues()
    {
        $result = $this->getValues();

        $result['root_category'] = $result['search_category']
            ? $result['search_category'][0]
            : NULL;

        $result['search_category'] = $result['search_category']
            ? $result['search_category'][count($result['search_category'])-1]
            : NULL;

        if (isset($result['search_keywords'])
            && '' != $result['search_keywords']
        ) {
            $keywords = explode("\n", $result['search_keywords']);
            foreach ($keywords as $i=>$keyword) {
                $keywords[$i] = trim($keyword);
            }
            $result['search_keywords'] = $keywords;
        } else {
            $result['search_keywords'] = array();
        }

        // Replace posts_per_week with schedule
        if ('immediately' != $result['publish_type']) {
            $schedule = $result['schedule'] =
            Asg_Application::getConfig()->schedules->$result['posts_per_week'];
            if (!$schedule) {
                throw new Exception('Schedule not found');
            }
            $result['schedule'] = $schedule->toArray();
        }
        unset($result['posts_per_week']);

        // Add selected - emulate asin mode
        if ($this->getElement('select_items')) {
            $result['keywords_type'] = 'asin';
            $result['search_keywords'] = $this->getValue('select_items');
            unset($result['select_items']);
        }

        return $result;
    }

    public function hasCredentials()
    {
        foreach (Asg_Application::getModel('AffiliateCore')
            ->getCredentialFields() as $field
        ) {
            $value = $this->getValue($field);
            if (!$value) {
                return FALSE;
            }
        }

        return TRUE;
    }

    public function exportCredentials()
    {
        $result = array();

        foreach (Asg_Application::getModel('AffiliateCore')
            ->getCredentialFields() as $field
        ) {
            $result[$field] = $this->getValue($field);
        }

        return $result;
    }

    //
    // Getters and setters
    //

    public function getMarkets()
    {
        return $this->_markets;
    }

    public function getMarketCodes()
    {
        return array_keys($this->_markets);
    }

    public function getSearchCategoryFields()
    {
        return $this->_searchCategoryFields;
    }
}