<?php
require_once 'AsgZend/Form.php';
require_once 'AsgPumka/Filter/UrlEncode.php';

class Asg_Form_Options extends AsgZend_Form
{
    /**
    * @var Asg_Model_Template
    */
    protected $_markets;

    public function init()
    {
        /**
        * @var Asg_Model_AffiliateCore
        */
        $core = Asg_Application::getModel('AffiliateCore');
        $multiOptions = $core->getMultiOptions();

        $this->setDisableTranslator(TRUE);

        $this->addElement('checkbox', 'viewonamazon_visible', array(
            'label' => 'Show "View on Amazon" button',
        ));

        $this->addElement('file', 'button_image', array(
            'validators' => array(
                array('Count', TRUE, 1),
                array('Extension', TRUE, array('jpg', 'jpeg', 'png', 'gif')),
                array('FilesSize', TRUE, array('max' => '1MB')),
                //array('ImageSize', TRUE, array('maxheight' => 400, 'maxwidth' => 1024)),
            ),
            'accept' => 'image/jpeg,image/png,image/gif',
            'ignore' => TRUE,
            'label' => 'Button image',
        ));

        $this->addElement('checkbox', 'button_image_reset', array(
            'label' => 'Reset to default',
            '-help' => FALSE,
        ));

        $this->addElement('text', 'button_text', array(
            'filters' => array(
                'StripNewlines',
                'StringTrim',
            ),
            'label' => 'Button text',
        ));

        $this->addElement('checkbox', 'addtocart_visible', array(
            'label' => 'Show 90-day-cookie, add-to-cart button',
        ));

        $this->addElement('file', 'addtocart_image', array(
            'validators' => array(
                array('Count', TRUE, 1),
                array('Extension', TRUE, array('jpg', 'jpeg', 'png', 'gif')),
                array('FilesSize', TRUE, array('max' => '1MB')),
                //array('ImageSize', TRUE, array('maxheight' => 400, 'maxwidth' => 1024)),
            ),
            'accept' => 'image/jpeg,image/png,image/gif',
            'ignore' => TRUE,
            'label' => 'Add-to-cart button image',
        ));

        $this->addElement('checkbox', 'addtocart_image_reset', array(
            'label' => 'Reset to default',
            '-help' => FALSE,
        ));

        $this->addElement('text', 'addtocart_text', array(
            'filters' => array(
                'StripNewlines',
                'StringTrim',
            ),
            'label' => 'Button text',
        ));

        $this->addElement('select', 'eye_candy_align', array(
            'required' => TRUE,
            'multiOptions' => array(
                'left' => 'Left',
                'right' => 'Right',
            ),
            'label' => 'Eye candy align',
        ));

        $this->addElement('checkbox', 'show_button_index', array(
            'label' => "Show buttons on the blog's index pages (like the homepage and category pages)",
        ));

        $this->addElement('checkbox', 'show_button_bottom', array(
            'label' => "Show buttons at the bottom of the post",
        ));

        // Rewiews settings

        /*
        $this->addElement('checkbox', 'show_reviews', array(
            'label' => "Show customer reviews",
        ));
        */
        $this->addElement('radio', 'show_reviews_rev2', array(
            'label' => "Customer Reviews",
            'multiOptions' => array(
                'hide' => 'Don\'t show customer reviews',
                'iframe' => 'Show Amazon-approved reviews frame',
                'embed' => 'Embed reviews into post content',
            ),
            'required' => TRUE,
        ));

        $this->addElement('select', 'min_review_rating', array(
            'multiOptions' => $multiOptions['min_review_rating'],
            'label' => 'Minimum review rating',
        ));

        //
        // Market fields
        //

        $defaultMarket = Asg_Application::getModel('Options')->getValue('amazon_market');
        $this->_markets = Asg_Application::getModel('Amazon_Market')->getList();
        foreach ($this->_markets as $marketCode=>$marketLabel) {
            $this->addElement('text', "{$marketCode}_affiliate_id", array(
                'filters' => array(
                    'StripNewlines',
                    'StringTrim',
                    'Null',
                ),
                'validators' => array(
                    array('Regex', TRUE,
                        array(
                            'pattern' => '~^[a-zA-Z0-9\-\.]+$~u',
                            'messages' => array(
                                'regexNotMatch' => "'%value%' contains invalid characters",
                            ),
                        ),
                    )
                ),
                'required' => $marketCode == $defaultMarket,
                'label' => "{$marketLabel} Associate ID",
                '-help' => FALSE,
            ));
        }

        //
        // Link settings
        //

        $this->addElement('text', 'amazon_key', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
            ),
            'label' => 'Access Key ID',
        ));

        $this->addElement('password', 'amazon_secret_key', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
            ),
            'label' => 'Secret Access Key',
        ));

        $this->addElement('text', 'tracking_id', array(
            'filters' => array(
                'StripNewlines',
                'StringTrim',
                array('Null', array('string')),
            ),
            'label' => 'Tracking ID',
        ));

        $this->addElement('checkbox', 'cloak_enabled', array(
            'label' => 'Cloak Affiliate Links',
        ));

        $this->addElement('text', 'cloak_dir', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
                new AsgPumka_Filter_UrlEncode,
                array('Null', array('string')),
            ),
            'label' => 'Cloaking  Directory',
            '-help' => FALSE,
        ));
    }

    public function loadDefaults()
    {
        /**
        * @var Asg_Model_Options
        */
        $options = Asg_Application::getModel('Options');

        /**
        * @var Asg_Model_AffiliateCore
        */
        $defaults = Asg_Application::getModel('AffiliateCore')->getDefaults();

        foreach ($this->getElements() as $elem) {
            $name = $elem->getName();
            $elem->setValue(
                $options->getValue($name,
                isset($defaults[$name])? $defaults[$name] : NULL));
        }

        return $this;
    }

    public function isValid($data)
    {
        $this->setDefaults($data);

        if (!isset($data['min_review_rating'])) {
            $data['min_review_rating'] = $this->getValue('min_review_rating');
        }

        return parent::isValid($data);
    }

    public function saveDefaults()
    {
        $ignoreFields = array(
            'button_image',
            'button_image_reset',
            'addtocart_image',
            'addtocart_image_reset',
        );

        $options = Asg_Application::getModel('Options');

        foreach ($this->getElements() as $elem) {
            if (!in_array($elem->getName(), $ignoreFields)) {
               $options->setValue($elem->getName(), $elem->getValue());
            }
        }

        foreach (array('button', 'addtocart') as $name) {
            if ($this->getValue("{$name}_image_reset")) {
                Asg_Application::getModel('Template')->resetButtonImage($name);
            } else {
                $upload = $this->getElement("{$name}_image");
                if ($upload->isUploaded()) {
                    try {
                        Asg_Application::getModel('Template')->uploadButtonImage($upload, $name);
                    } catch (Asg_Model_Template_Exception $e) {
                        require_once ASG_APPLICATION_PATH . '/forms/Options/Exception.php';
                        throw new Asg_Form_Options_Exception(
                            $e->getMessage(), "{$name}_image");
                    }
                }
            }
        }

        return $this;
    }

    //
    // Getters and setters
    //

    public function getMarkets()
    {
        return $this->_markets;
    }

    public function getMarketCodes()
    {
        return array_keys($this->_markets);
    }
}