<?php
require_once ASG_APPLICATION_PATH . '/models/AbstractModel.php';

class Asg_Model_AffiliateCore extends Asg_Model_AbstractModel
{
    protected $_service;

    // Session keys
    protected $_sessOutput = Asg_WpPlugin::PREFIX;

    public function getCategories($parent=NULL, $options=array())
    {
        $this->setOptions($options, FALSE); // getting categories doesn't require associate ID
        return $this->getService()->getAdapter()->getCategories($parent);
    }

    public function generate($options=array())
    {
        $this->setOptions($options)
            ->getService()->generate();

        return $this;
    }

    public function preview($options=array())
    {
        set_time_limit(600);

        return $this->setOptions($options)
            ->getService()->getAdapter()->getItems($options['max_items'], TRUE);
    }

    public function refresh($postId, $itemId, $options=array())
    {
        return $this->setOptions($options)
            ->getService()->refresh($postId, $itemId);
    }

    public function collectItemData($itemId, $market)
    {
        $this->setOptions(array('amazon_market' => $market));

        try {
            $result = $this->getService()->getScraper('amazonApi')
                ->collectItemData($itemId);
        } catch (Asg_Service_Affiliate_FatalException $e) {
            $result = NULL;
        } catch (Asg_Service_Affiliate_Exception $e) {
            $result = NULL;
        }

        return $result;
    }

    public function getAmazonDomain()
    {
        $market = Asg_Application::getModel('Options')->getValue('amazon_market');
        return Asg_Application::getConfig()->amazon->marketInfo->$market->domain;
    }

    public function setOptions(array $options=array(), $useAffiliateId=TRUE)
    {
        $optionsModel = Asg_Application::getModel('Options');

        // Load credentials if they are missing
        foreach ($this->getCredentialFields() as $field) {
            if (!isset($options[$field])) {
                $options[$field] = $optionsModel->getValue($field);
            }
        }

        // Set associate ID
        if ($useAffiliateId && !isset($options['affiliate_id'])) {
            if (!isset($options['amazon_market'])) {
                throw new Exception('Amazon market not specified');
            }

            $value = $optionsModel->getValue("{$options['amazon_market']}_affiliate_id");
            if ('' == $value) {
                throw new Exception("Associate ID for market '{$options['amazon_market']}' is not not specified");
            }

            $options['affiliate_id'] = $value;
        }

        // Multiply prices x100 (@todo: research)
        foreach (array('price_min', 'price_max') as $key) {
            if (isset($options[$key])) {
                $options[$key] *= 100;
            }
        }

        $this->getService()->setOptions($options);

        return $this;
    }

    //
    // Output handling
    //

    public function isActive()
    {
        if ((!isset($_SESSION[$this->_sessOutput]))
            || (isset($_SESSION[$this->_sessOutput]['command'])
            && 'terminate' == $_SESSION[$this->_sessOutput]['command']))
        {
            $result = FALSE;
        } else {
            $result = TRUE;
        }

        return $result;
    }

    public function getOutput()
    {
        return @$_SESSION[$this->_sessOutput];
    }

    public function sendCommand($command)
    {
        $_SESSION[$this->_sessOutput]['command'] = $command;
    }

    public function terminate()
    {
        $this->sendCommand('terminate');
    }

    public function clearOutput()
    {
        unset($_SESSION[$this->_sessOutput]);
    }

    //
    // Getters & setters
    //

    public function getService()
    {
        return Asg_Application::getService('Affiliate');
    }

    public function getDefaults()
    {
        return Asg_Application::getConfig()->affiliate->options->toArray();
    }

    public function getConstraints()
    {
        return Asg_Application::getConfig()->affiliate->constraints->toArray();
    }

    public function getMultiOptions()
    {
        $config = Asg_Application::getConfig();
        $result = $config->affiliate->multi_options->toArray();

        //Add posts_per_week options
        $postsPerWeek = array();
        foreach ($config->schedules->toArray() as $key=>$val) {
            $postsPerWeek[$key] = $key;
        }
        $result['posts_per_week'] = $postsPerWeek;

        return $result;
    }

    public function getCredentialFields()
    {
        return Asg_Application::getConfig()->affiliate->credentials->toArray();
    }
}