<?php
require_once ASG_APPLICATION_PATH . '/models/AbstractModel.php';

class Asg_Model_Amazon_Data extends Asg_Model_AbstractModel
{
    protected $_data;
    protected $_postId;

    protected $_defaultImageSize = 'MediumImage';

    protected function _checkData()
    {
        if (!is_array($this->_data)) {
            throw new Exception('Amazon data not initialized');
        }
    }

    public function extractPrice()
    {
        $this->_checkData();

        // Try offer sale price
        if (isset($this->_data['Offers']['Offer']['OfferListing']['SalePrice']['FormattedPrice'])) {
            return $this->_data['Offers']['Offer']['OfferListing']['SalePrice']['FormattedPrice'];
        }

        // Try offer price
        if (isset($this->_data['Offers']['Offer']['OfferListing']['Price']['FormattedPrice'])) {
            return $this->_data['Offers']['Offer']['OfferListing']['Price']['FormattedPrice'];
        }

        // Try lowest variation price
        if (isset($this->_data['VariationSummary']['LowestSalePrice']['FormattedPrice'])) {
            return $this->_data['VariationSummary']['LowestSalePrice']['FormattedPrice'];
        }

        // Try list price
        if (isset($this->_data['ItemAttributes']['ListPrice']['FormattedPrice'])) {
            return $this->_data['ItemAttributes']['ListPrice']['FormattedPrice'];
        }

        return NULL;
    }

    protected function extractListPrice()
    {
        $this->_checkData();

        // Try list price
        if (isset($this->_data['ItemAttributes']['ListPrice']['FormattedPrice'])) {
            return $this->_data['ItemAttributes']['ListPrice']['FormattedPrice'];
        }

        // Try lowest variation price
        if (isset($this->_data['VariationSummary']['LowestSalePrice']['FormattedPrice'])) {
            return $this->_data['VariationSummary']['LowestSalePrice']['FormattedPrice'];
        }

        // Try offer price
        if (isset($this->_data['Offers']['Offer']['OfferListing']['Price']['FormattedPrice'])) {
            return $this->_data['Offers']['Offer']['OfferListing']['Price']['FormattedPrice'];
        }

        return NULL;
    }

    public function extractProductUrl()
    {
        return Asg_Application::getModel('Amazon_Links')
            ->setPost(get_post($this->_postId))
            ->getDetailsUrl();
    }

    public function extractImageUrl($size = NULL)
    {
        $result = $this->extractImageData($size);

        return $result? $result['URL']: NULL;
    }

    public function extractImageData($size = NULL)
    {
        $this->_checkData();

        if (is_null($size)) {
            $size = $this->_defaultImageSize;
        }

        // Try normal image
        if (isset($this->_data[$size])) {
            return $this->_data[$size];
        }

        // Try first image from variants
        if (isset($this->_data['ImageSets']['ImageSet'][0][$size])) {
            return $this->_data['ImageSets']['ImageSet'][0][$size];
        }

        return NULL;
    }

    public function extractElement(array $path)
    {
        $this->_checkData();

        return $this->_traverseArray($this->_data, $path);
    }

    protected function _traverseArray(array $array, array &$path)
    {
        $key = array_shift($path);

        if (is_null($key)) {
            return NULL;
        }

        if (!isset($array[$key])) {
            return NULL;
        }

        $elem = $array[$key];

        if (!$path) {
            return $elem;
        }

        if (!is_array($elem)) {
            return NULL;
        }

        return $this->_traverseArray($elem, $path);
    }

    //
    // Getters & setters
    //

    public function setData($value)
    {
        $this->_data = $value;

        return $this;
    }

    public function getData()
    {
        return $this->_data;
    }

    public function setPostId($value)
    {
        $this->_postId = $value;

        return $this;
    }
}