<?php
require_once ASG_APPLICATION_PATH . '/models/AbstractModel.php';

class Asg_Model_Amazon_Links extends Asg_Model_AbstractModel
{
    protected $_post;
    protected $_postId;

    protected $_defaultCloak;
    protected $_cloak;

    protected $_trackingId;

    public function __construct()
    {
        $options = Asg_Application::getModel('Options');

        $this->_trackingId = $options->getValue('tracking_id');
        $this->_defaultCloak = (bool) $options->getValue('cloak_enabled');
    }

    public function getDetailsUrl($customMarket=NULL)
    {
        $this->_checkPost();

        if ($this->_isCloakEnabled()) {
            return $this->_cloakUrlBasic('view', $customMarket);
        }

        $metaKey = !is_null($customMarket)
            ? "{$customMarket}_amazon_data"
            : 'amazon_data';
        $data = Asg_Application::getModel('PostMeta')
            ->setPostId($this->_postId)
            ->getValue($metaKey);
        if (!is_array($data) || !isset($data['DetailPageURL'])) {
            return NULL;
        }

        $result = $data['DetailPageURL'];

        if ('' != $this->_trackingId) {
            $result = add_query_arg('t', urlencode($this->_trackingId), $result);
        }

        return $result;
    }

    public function getCartUrl($customMarket=NULL)
    {
        $this->_checkPost();

        if ($this->_isCloakEnabled()) {
            return $this->_cloakUrlBasic('add-to-cart', $customMarket);
        }

        $options = Asg_Application::getModel('Options');
        $meta = Asg_Application::getModel('PostMeta')
            ->setPostId($this->_postId);

        $market = is_null($customMarket)
            ? $meta->getValue('amazon_market')
            : $customMarket;

        $params = array(
            'AssociateTag' => $options->getValue("{$market}_affiliate_id"),
            'SubscriptionId' => $options->getValue('amazon_key'),
            'ASIN.1' => $meta->getValue('item_id'),
            'Quantity.1' => 1,
        );
        if ('' != $this->_trackingId) {
            $params['t'] = $this->_trackingId;
        }

        $domain = Asg_Application::getConfig()->amazon->marketInfo->$market->domain;

        return 'http://' . $domain . '/gp/aws/cart/add.html?'
            . http_build_query($params, '_', '&');
    }

    public function decloakPath(array $path)
    {
        $slug = $path[0];

        $res = get_posts(array(
            'name' => $slug,
            'post_status' => 'any',
        ));

        if (!$res) {
            return NULL;
        }

        $post = array_shift($res);
        $this->setPost($post);

        $customMarket = isset($path[2])? strtoupper($path[2]) : NULL;

        $oldCloak = $this->_cloak;
        $this->setCloak(FALSE);

        $result = 'add-to-cart' == $path[1]
            ? $this->getCartUrl($customMarket)
            : $this->getDetailsUrl($customMarket);

        $this->setCloak($oldCloak);

        return $result;
    }

    protected function _cloakUrlBasic($type='view', $customMarket=NULL)
    {
        $path = array($type);
        if (!is_null($customMarket)) {
            $path[] = strtolower($customMarket);
        }

        return $this->_cloakUrl($path);
    }

    protected function _cloakUrl($extraCloakDirs=NULL)
    {
        global $wp_rewrite;

        $path = array(untrailingslashit(site_url()));

        if (!$wp_rewrite->using_mod_rewrite_permalinks()) {
            $path[] = $wp_rewrite->index;
        }

        $path[] = Asg_Application::getModel('Options')->getValue('cloak_dir');
        $path[] = $this->_post->post_name;

        if ($extraCloakDirs) {
            $path = array_merge($path, (array) $extraCloakDirs);
        }

        $result = implode('/', $path);

        return $result;
    }

    protected function _isCloakEnabled()
    {
        return is_null($this->_cloak)
            ? $this->_defaultCloak
            : $this->_cloak;
    }

    //
    // Setters
    //

    public function setPost($value)
    {
        $this->_post = $value;
        $this->_postId = $value->ID;

        return $this;
    }

    public function setCloak($value)
    {
        $this->_cloak = $value;

        return $this;
    }

    protected function _checkPost()
    {
        if (!$this->_post) {
            throw new Exception('Post not specified');
        }
    }
}