<?php
require_once ASG_APPLICATION_PATH . '/models/AbstractModel.php';

class Asg_Model_Amazon_Market extends Asg_Model_AbstractModel
{
    const DEFAULT_MARKET_CODE = 'US';

    protected $_code = self::DEFAULT_MARKET_CODE;
    protected $_config = array();

    protected $_suggestedMarket;

    public function getConfig()
    {
        if (!isset($this->_config[$this->_code])) {
            require_once ASG_LIBRARY_PATH . '/AsgZend/Config/Ini.php';
            $this->_config[$this->_code] = new AsgZend_Config_Ini(ASG_APPLICATION_PATH . '/configs/amazon.ini', $this->_code);
        }

        return $this->_config[$this->_code];
    }

    public function setCode($value)
    {
        $this->_code = $value;
        return $this;
    }

    public function getList($supportedOnly=FALSE)
    {
        $result = Asg_Application::getConfig()->amazon->markets->toArray();
        if ($supportedOnly) {
            $options = Asg_Application::getModel('Options');

            foreach ($result as $marketCode=>$label) {
                $value = $options->getValue("{$marketCode}_affiliate_id");
                if ('' == $value) {
                    unset($result[$marketCode]);
                }
            }
        }

        return $result;
    }

    public function getCodeList($supportedOnly=FALSE)
    {
        return array_keys($this->getList($supportedOnly));
    }

    public function suggest()
    {
        if (is_null($this->_suggestedMarket)) {
            $this->_suggestedMarket = '';

            $supported = $this->getCodeList(TRUE);

            $countryCode = Asg_Application::loadLibrary('AsgPumka_GeoIp')
                ->recognizeCountryCode();

            if (!is_null($countryCode)) {
                $cfg = Asg_Application::getConfig()->amazon->countryMarket;
                if (isset($cfg->$countryCode)) {
                    $marketCode = (string) $cfg->$countryCode;

                    if (in_array($marketCode, $supported)) {
                        $this->_suggestedMarket = $marketCode;
                    }
                }
            }
        }


        return $this->_suggestedMarket;
    }
}