<?php
require_once ASG_APPLICATION_PATH . '/models/AbstractModel.php';

class Asg_Model_Cache extends Asg_Model_AbstractModel
{
    const OPTION_NAME = 'cache';
    const DEFAULT_TTL = 86400;

    protected $_cache;

    public function __construct()
    {
        $this->load();
    }

    public function getValue($key)
    {
        return isset($this->_cache[$key])
            ? $this->_cache[$key]['value']
            : NULL;
    }

    public function setValue($key, $value, $ttl = self::DEFAULT_TTL)
    {
        $this->_cache[$key] = array(
            'expire' => time() + $ttl,
            'value' => $value,
        );

        return $this;
    }

    public function load()
    {
        $this->_cache = Asg_Application::getModel('Options')
            ->getValue(self::OPTION_NAME, array());

        if (!is_array($this->_cache)) {
            $this->_cache = array();
        }

        $now = time();
        foreach ($this->_cache as $key=>$item) {
            if (!isset($item['expire']) || $item['expire'] < $now) {
                unset($this->_cache[$key]);
            }
        }

        return $this;
    }

    public function save()
    {
        Asg_Application::getModel('Options')
            ->setValue(self::OPTION_NAME, $this->_cache);

        return $this;
    }
}