<?php
require_once ASG_APPLICATION_PATH . '/models/AbstractModel.php';

class Asg_Model_Details extends Asg_Model_AbstractModel
{
    protected $_types = array('product_details', 'technical_details');
    protected $_details = array();

    public function prepare($data)
    {
        foreach ($this->_types as $type) {
            $this->_details[$type] = NULL;

            $keys = array();
            $values = array();

            if (isset($data['asg_' . $type]['key']) && is_array($data['asg_' . $type]['key'])) {
                $keys = $data['asg_' . $type]['key'];
            }

            if (isset($data['asg_' . $type]['val']) && is_array($data['asg_' . $type]['val'])) {
                $values = $data['asg_' . $type]['val'];
            }

            $keys = $this->_filterKeys($keys);
            $values = $this->_filterValues($values);

            $length = min(count($keys), count($values));

            for ($i = 0; $i < $length; $i++) {
                if ('' == $values[$i]) {
                    continue;
                }
                if ('' == $keys[$i]) {
                    $this->_details[$type][] = $values[$i];
                } else {
                    $this->_details[$type][$keys[$i]]= $values[$i];
                }
            }

        }
        return $this;
    }

    public function save($postId)
    {
        $meta = Asg_Application::getModel('PostMeta')->setPostId($postId);

        foreach ($this->_types as $type) {
            if (isset($this->_details[$type])) {
                $meta->setValue($type, $this->_details[$type]);
            } else {
                $meta->unsetValue($type);
            }

        }
        return $this;
    }

    protected function _filterKeys($data)
    {
        return $this->_filterArray($data);
    }

    protected function _filterValues($data)
    {
        $result = array();

        foreach ($data as $val){
            $arr = explode("\n", trim($val));
            if (1 == count($arr)) {
                $result[] = $this->_filterString($val);
            } else {
                $result[] = $this->_filterArray($arr, 'DROP_EMPTY');
            }
        };

        return $result;
    }

    protected function _filterArray($data, $option = '')
    {
        $result = array();

        foreach ($data as $str) {
            $str = $this->_filterString($str);
            if (('DROP_EMPTY' == $option) && ('' == $str)) {
                continue;
            }
            $result[] = $str;
        };

        return $result;
    }

    protected function _filterString($str) {
        $str = str_replace(array("\n", "\r"), '', $str);
        $str = trim($str);
        return $str;
    }
}
