<?php
    require_once ASG_APPLICATION_PATH . '/models/AbstractModel.php';

    class Asg_Model_Template extends Asg_Model_AbstractModel
    {
    /**
    * @var Asg_Model_Options
    */
    protected $_optionsModel;
    protected $_linksModel;

    protected $_shortcodePrefix = 'ag';
    protected $_shortcodes = array(
        'top',
        'before-rating',
        'before-price',
        'after-technical-details',
        'after-product-details',
        'after-reviews',
        'bottom',
    );

    public function __construct()
    {
        $this->_optionsModel = Asg_Application::getModel('Options');
    }

    public function prepareTemplateVars(array &$vars)
    {
        $this->_linksModel = Asg_Application::getModel('Amazon_Links')
            ->setPost($vars['post']);

        if (!isset($vars['single'])) {
            $vars['single'] = is_singular();
        }

        /**
        * @var Asg_Model_Amazon_Market
        */
        $marketModel = Asg_Application::getModel('Amazon_Market');

        $markets = $marketModel->getList();

        $market = isset($vars['amazon_market']) && isset($markets[$vars['amazon_market']])
        ? $vars['amazon_market']
        : (string)Asg_Application::getConfig()->option_defaults->amazon_market;
        $vars['amazon_market'] = $market;

        $now = time();
        if (!is_array($vars['amazon_data'])
            || ((
                !$vars['update_timestamp']
                || $now > ($vars['update_timestamp'] + $this->_optionsModel->getValue('refresh_interval'))
                )
                && (
                    !isset($vars['last_refresh_attempt'])
                    || !$vars['last_refresh_attempt']
                    || $now > ($vars['last_refresh_attempt'] + $this->_optionsModel->getValue('refresh_attempt_interval'))
            ))
        ) {
            $options = array();

            $options['amazon_market'] =
            isset($vars['amazon_market']) && isset($markets[$vars['amazon_market']])
            ? $vars['amazon_market']
            : (string)Asg_Application::getConfig()->option_defaults->amazon_market;

            try {
                $refresh = Asg_Application::getModel('AffiliateCore')->refresh(
                    $vars['post_id'], $vars['item_id'], $options);
                $vars = array_merge($vars, $refresh);
                echo "<!-- Amazon data refresh successful -->";
            } catch (Exception $e) {
                echo "<!-- refresh failed: {$e->getMessage()} -->";
                if (!is_array($vars['amazon_data'])) {
                    return FALSE;
                }
            }
        }

        $this->_prepareShortcodes($vars);
        $this->_preparePrice($vars);
        $this->_prepareOptions($vars);
        $this->_prepareProductImage($vars);
        $this->_prepareUrls($vars);

        $vars['eyeCandy'] = (object)($this->getEyeCandy());

        // Alternative market

        // test IP:
        //
        //$_SERVER['REMOTE_ADDR'] = '212.58.251.195';

        $altMarket = $marketModel->suggest();
        if ('' != $altMarket && $altMarket != $market) {
            $meta = Asg_Application::getModel('PostMeta')
            ->setPostId($vars['post_id']);

            $altData = $meta->getValue("{$altMarket}_amazon_data");
            $altUpdateTimestamp = $meta->getValue("{$altMarket}_update_timestamp");

            if (!is_array($altData) || !$altUpdateTimestamp
                || $now > ($altUpdateTimestamp + $this->_optionsModel->getValue('refresh_interval'))
            ) {
                $altData = Asg_Application::getModel('AffiliateCore')
                ->collectItemData($vars['item_id'], $altMarket);

                if (!$altData) {
                    $altData = array('not_available' => TRUE);
                }

                $altUpdateTimestamp = $now;

                $meta->setValue("{$altMarket}_amazon_data", $altData);
                $meta->setValue("{$altMarket}_update_timestamp", $now);
            }
            if (!isset($altData['not_available'])) {
                $vars['alt'] = array(
                    'market' => $altMarket,
                    // 'amazon_data' => $altData,
                    'price' => $this->_extractPrice($altData),
                    'url' => $this->_linksModel->getDetailsUrl($altMarket),
                    'updated' => $this->_formatDate($altUpdateTimestamp),
                );
            }
        }

        return TRUE;
    }

    public function getEyeCandy()
    {
        $result = array();

        if ($this->_optionsModel->getValue('viewonamazon_visible')) {
            $result['button'] = $this->_getEyeCandyButton('button');
        }
        if ($this->_optionsModel->getValue('addtocart_visible')) {
            $result['addtocart'] = $this->_getEyeCandyButton('addtocart');
        }

        $options = array(
            'show_button_index'  => 'showIndex',
            'eye_candy_align'    => 'align',
            'show_button_bottom' => 'showBottom',
        );
        foreach ($options as $key=>$val) {
            $result[$val] = $this->_optionsModel->getValue($key);
        }

        return $result;
    }

    protected function _getEyeCandyButton($name = 'button')
    {
        $result = $this->getButtonImage($name);
        $result['text'] = $this->_optionsModel->getValue("{$name}_text");

        return $result;
    }

    public function getButtonImage($name = 'button')
    {
        $result = $this->_optionsModel->getValue("{$name}_image");
        if (!$result
            || !is_array($result)
            || !isset($result['file'])
            || !isset($result['url'])
            || !isset($result['width'])
            || !isset($result['height'])
        ) {
            $result = $this->_getDefaultButtonImage();
        }

        return $result;
    }

    protected function _getDefaultButtonImage()
    {
        $plugin = Asg_WpPlugin::getInstance();
        $result = array(
            'file' => $plugin->getPluginPath() . '/template/images/amazon-button.png',
            'url' => $plugin->getPluginBaseUrl() . '/template/images/amazon-button.png',
            'width' => 160,
            'height' => 27,
            'type' => 'image/png',
            'default' => TRUE,
        );

        return $result;
    }

    public function uploadButtonImage(AsgZend_Form_Element_File $upload,
        $name = 'button')
    {
        $fieldName = $upload->getName();

        $info = $upload->getTransferAdapter()->getFileInfo();
        $info = $info[$fieldName];
        @$sizeInfo = getimagesize($info['tmp_name']);
        if (FALSE === $sizeInfo) {
            require_once ASG_APPLICATION_PATH . '/models/Template/Exception.php';
            throw new Asg_Model_Template_Exception('Invalid image file');
        }

        $fileInfo = wp_handle_upload($_FILES[$fieldName],
            array('test_form' => FALSE));

        if (isset($fileInfo['error'])) {
            require_once ASG_APPLICATION_PATH . '/models/Template/Exception.php';
            throw new Asg_Model_Template_Exception(
                "Error uploading file: {$fileInfo['error']}");
        }

        // Add size data
        $fileInfo['width'] = $sizeInfo[0];
        $fileInfo['height'] = $sizeInfo[1];

        //Save option
        $this->_optionsModel->setValue("{$name}_image", $fileInfo);
    }

    public function resetButtonImage($name = 'button')
    {
        $this->_optionsModel->unsetValue("{$name}_image");
    }

    protected function _preparePrice(&$vars)
    {
        $vars['price'] = $this->_extractPrice($vars['amazon_data']);
        $vars['price']['date'] = $this->_formatDate(
            isset($vars['update_timestamp'])
            ? $vars['update_timestamp']
            : NULL);
    }

    protected function _extractPrice($data)
    {
        $result = array();

        if (isset($data['ItemAttributes']['ListPrice'])) {
            $listPrice = $data['ItemAttributes']['ListPrice']['FormattedPrice'];
            $listPriceAmount = $data['ItemAttributes']['ListPrice']['Amount'];
        } else {
            $listPriceAmount = NULL;
            $listPrice = NULL;
        }

        $offer = NULL;
        if (isset($data['Offers']['Offer']['OfferListing']['Price']['Amount'])) {
            $offer = $data['Offers']['Offer']['OfferListing'];
            if ($offer['Price']['Amount'] > $listPriceAmount) {
                $listPrice = $offer['Price']['FormattedPrice'];
                $listPriceAmount = $offer['Price']['Amount'];
                if (!isset($offer['SalePrice'])) {
                    $offer = NULL;
                }
            }
        }

        if (is_null($listPrice)
            && isset($data['VariationSummary']['LowestSalePrice'])
        ) {
            $listPrice =  $data['VariationSummary']['LowestSalePrice']['FormattedPrice'];
            $listPriceAmount = $data['VariationSummary']['LowestSalePrice']['Amount'];
        }

        if (isset($data['VariationSummary']['LowestPrice']['FormattedPrice'])
            && $data['VariationSummary']['HighestPrice']['FormattedPrice']
        ) {
            $lowestPrice = $data['VariationSummary']['LowestPrice']['FormattedPrice'];
            $highestPrice = $data['VariationSummary']['HighestPrice']['FormattedPrice'];
            if (($lowestPrice != $highestPrice) && ('' != $highestPrice)) {
                $variationPrice =  $lowestPrice . ' - ' . $highestPrice;
            } else {
                $variationPrice = $lowestPrice;
            }

            if (is_null($listPrice)
                || $listPriceAmount < $data['VariationSummary']['HighestPrice']['Amount']
            ) {
                $listPrice = $variationPrice;
                $listPriceAmount = NULL;
            } else {
                // Emulate offer
                $offer = array(
                    'Price' => array(
                        'FormattedPrice' => $variationPrice,
                    )
                );
            }
        }

        if (isset($data['OfferSummary'])
            && 0 == $data['OfferSummary']['TotalNew']
            && 0 == $data['OfferSummary']['TotalUsed']
            && 0 == $data['OfferSummary']['TotalCollectible']
            && 0 == $data['OfferSummary']['TotalRefurbished']
            && isset($data['ItemAttributes']['ListPrice'])
        ) {
            $unavailable = TRUE;
            // Check variations
            if (isset($data['Variations']['Item'])) {
                $variations = isset($data['Variations']['Item']['ASIN'])
                    ? array($data['Variations']['Item'])
                    : $data['Variations']['Item'];
                foreach ($variations as $variation) {
                    if (isset($variation['Offers']['Offer']) || isset($variation['Offer'])) {
                        $unavailable = FALSE;
                        break;
                    }
                }
            }

            if ($unavailable) {
                $result['unavailable'] = TRUE;
            }
        }

        $result['listPrice'] = !is_null($listPrice)? $listPrice : 'N/A';
        $result['offer'] = $offer;

        return $result;
    }

    protected function _formatDate($timestamp=NULL)
    {
        if (!$timestamp) {
            return 'unknown date';
        }

        return date(get_option('date_format') . ' ' .  get_option('time_format'),
            $timestamp);
    }

    protected function _prepareOptions(&$vars)
    {
        $options = array(
            'show_reviews_rev2',
            'min_review_rating',
        );
        foreach ($options as $key) {
            $vars['options'][$key] = $this->_optionsModel->getValue($key);
        }
    }

    protected function _prepareShortcodes(&$vars) {
        foreach ($this->_shortcodes as $shorcode) {
            $shorcode = $this->_shortcodePrefix . '-' . $shorcode;
            $str = '[\s\t\n\r]*';
            $pattern = "~{$str}\[{$shorcode}\](.*?)\[/{$shorcode}\]{$str}~isu";

            if (preg_match($pattern, $vars['content'], $matches)) {
                $vars['content'] = str_replace($matches[0], '', $vars['content']);
                $vars['shortcodes'][$shorcode] = trim($matches[1]);
            }
        }
    }

    protected function _prepareProductImage(&$vars)
    {
        $result = NULL;

        if (isset($vars['amazon_data']['MediumImage'])) {
            $result = $vars['amazon_data']['MediumImage'];
        } else if (isset($vars['amazon_data']['ImageSets']['ImageSet']['MediumImage'])) {
            $result = $vars['amazon_data']['ImageSets']['ImageSet']['MediumImage'];
        } else if (isset($vars['amazon_data']['ImageSets']['ImageSet'][0]['MediumImage'])) {
            $result = $vars['amazon_data']['ImageSets']['ImageSet'][0]['MediumImage'];
        } else if (isset($vars['amazon_data']['Variations']['Item'][0]['MediumImage'])) {
            $result = $vars['amazon_data']['Variations']['Item'][0]['MediumImage'];
        } else if (isset($vars['amazon_data']['Variations']['Item'][0]['ImageSets']['ImageSet']['MediumImage'])) {
            $result = $vars['amazon_data']['Variations']['Item'][0]['ImageSets']['ImageSet']['MediumImage'];
        }

        $vars['product_image'] = $result;
    }

    protected function _prepareUrls(&$vars)
    {
        $vars['details_url'] = $this->_linksModel->getDetailsUrl();
        $vars['cart_url'] = $this->_linksModel->getCartUrl();
    }
}