<?php
require_once ASG_APPLICATION_PATH . '/services/Affiliate/Helper.php';

class Asg_Service_Affiliate_Helper_Text
    extends Asg_Service_Affiliate_Helper
{
    public function htmlFragmentToText($text, $tags = NULL)
    {
        // Remove comments
        $this->_eraseFragments($text, '<!--', '-->');

        // Remove scripts and styles
        $this->_eraseFragments($text, '<script', '/script>');
        $this->_eraseFragments($text, '<style', '/style>');

        // Convert spaces and whitespace
        $this->_massReplace(array("\n", "\r", "\n", "\t", '&nbsp;', '&#160;', chr(194).chr(160)),
            ' ', $text);

        // Remove control chars
        $this->_massReplace(
            array(
                chr(0), chr(1), chr(2), chr(3), chr(4), chr(5), chr(6), chr(7),
                chr(8), chr(9), chr(10), chr(11), chr(12), chr(13), chr(14),
                chr(15), chr(16), chr(17), chr(18), chr(19), chr(20), chr(21),
                chr(22), chr(23), chr(24), chr(25), chr(26), chr(27), chr(28),
                chr(29), chr(30), chr(31),

                chr(194) . chr(128), chr(194) . chr(129), chr(194) . chr(130),
                chr(194) . chr(131), chr(194) . chr(132), chr(194) . chr(133),
                chr(194) . chr(134), chr(194) . chr(135), chr(194) . chr(136),
                chr(194) . chr(137), chr(194) . chr(138), chr(194) . chr(139),
                chr(194) . chr(140), chr(194) . chr(141), chr(194) . chr(142),
                chr(194) . chr(143), chr(194) . chr(144), chr(194) . chr(145),
                chr(194) . chr(146), chr(194) . chr(147), chr(194) . chr(148),
                chr(194) . chr(149), chr(194) . chr(150), chr(194) . chr(151),
                chr(194) . chr(152), chr(194) . chr(153), chr(194) . chr(154),
                chr(194) . chr(155), chr(194) . chr(156), chr(194) . chr(157),
                chr(194) . chr(158), chr(194) . chr(159),
            ),
            '', $text);

        //Ensure text is valid UTF8
        $text = wp_check_invalid_utf8($text, TRUE);

        // Convert line breaks
        $text = preg_replace('~\s*<br\b.*?>\s*~isu', "\n", $text);

        // Convert paragraphs
        $text = preg_replace('~<\s*\b/?(?:p|div|h\d)\b[^>]*?>~isu', "\n\n", $text);

        // Remove tags
        $text = strip_tags($text, $tags);

        // Decode
        $text = $this->htmlDecode($text);

        // Normalize spaces
        $text = preg_replace('~  +~isu', ' ', $text);

        // Remove blank lines
        $text = preg_replace('~ +\n~isu', '', $text);

        // Normalize line breaks
        $text = preg_replace('~\n{3,}~isu', "\n\n", $text);

        //Trim
        $text = trim($text);

        return $text;
    }

    public function htmlDecode($text)
    {
        $result = html_entity_decode($text, ENT_COMPAT, 'UTF-8');
        if (empty($result)) {
            $result = html_entity_decode($text, ENT_COMPAT);
        }

        return $result;
    }

    protected function _eraseFragments(&$subject, $start, $end)
    {
        for ($i=0; $i<200; $i++) {
            $posStart = strpos($subject, $start);
            if ($posStart===FALSE) break;
            $pos = $posStart+strlen($start);
            if ($pos >= strlen($subject)) {
                break;
            }
            $posEnd = strpos($subject, $end, $pos);
            if ($posEnd===FALSE) break;

            $subject = substr($subject, 0, $posStart) . substr($subject, $posEnd + strlen($end));
        }
    }

    protected function _massReplace(array $search, $replace, &$subject)
    {
        foreach ($search as $s) {
            $subject = str_replace($s, $replace, $subject);
        }
    }
}