<?php
require_once ASG_APPLICATION_PATH . '/services/Affiliate/Plugin/Publisher.php';

class Asg_Service_Affiliate_Plugin_Publisher_Wordpress
    extends Asg_Service_Affiliate_Plugin_Publisher
{
    protected $_publicationDelay = 0;  //removed in 2.0
    protected $_publicationStartTime = 36000; //10:00
    protected $_publicationRange = 28800; //8 hours

    protected $_publicationDeltaRatio = 0.25;  // Random post offset +/- as a part of post step

    protected $_addMoreTag = TRUE;

    protected $_publication;

    // Do not save these item fields as meta fields
    protected $_metaExclude = array(
        'text',
        'title',
        'description',
        'keywords',
    );

    public function init()
    {
        global $wpdb;

        // Workaround for connection timeout on HostGator
        $wpdb->query("set session wait_timeout=600");
    }

    public function publish(array &$item)
    {
        $text = $this->_htmlEscape($item['text']);
        if ($this->_addMoreTag) {
            $paragraphs = explode("\n\n", $text);
            if (count($paragraphs) > 2) {
                $text = '';
                $last = count($paragraphs) - 1;
                $i = 0;
                foreach($paragraphs as $paragraph) {
                    $text .= $paragraph;
                    if (1 == $i) {
                        $text .= "\n<!--more-->\n";
                    } elseif ($i < $last) {
                        $text .= "\n\n";
                    }
                    $i++;
                }
            }
        }

        $post = array();
        $post['post_content'] = $text;
        $post['post_title'] = $item['title'];
        if (isset($item['description'])) {
            $post['post_excerpt'] = $item['description'];
        }
        if (isset($item['keywords'])) {
            $post['tags_input'] = implode(', ', $item['keywords']);
        }
        $post['post_category'] = $this->getOption('post_category');

        $scheduleTime = $this->_getTime();
        if (!is_null($scheduleTime)) {
            $post['post_status'] = 'future';
            $post['post_date'] = date('Y-m-d H:i:s', $scheduleTime);
        } else {
            $post['post_status'] = 'publish';
        }

        // Write post
        $postId = wp_insert_post($post);
        if (is_a($postId, 'WP_Error')) {
            $this->throwError('Post creation failed: ' . $postId->get_error_message(), 'fatal');
        } else if (!$postId) {
            global $wpdb;
            if ('' != $wpdb->last_error) {
                $this->throwError('Post creation failed due to database error: ' . $wpdb->last_error, 'critical');
            }
            $this->throwError('Post creation failed due to unknown error.', 'critical');
        }

        // Save metadata
        $this->_saveMeta($postId, $item);

        return $this;
    }

    public function refresh($publicationId, array &$item)
    {
        $this->_saveMeta($publicationId, $item, TRUE);

        return $this;
    }

    public function refreshFailed($publicationId, array &$item)
    {
        $item['last_refresh_attempt'] = time();
        $meta = Asg_Application::getModel('PostMeta')->setPostId($publicationId);
        $meta->setValue('last_refresh_attempt', $item['last_refresh_attempt']);

        return $this;
    }

    protected function _saveMeta($postId, array &$item, $refresh=FALSE)
    {
        // Save metadata
        $meta = Asg_Application::getModel('PostMeta')->setPostId($postId);

        if (!$refresh) {
            $meta->setValue('version', Asg_WpPlugin::VERSION);
        }

        $item['update_timestamp'] = $item['last_refresh_attempt'] = time();

        $exclude = $this->_metaExclude;
        if ($refresh) {
            $exclude[] = 'item_id';
        }

        foreach ($item as $key => $val) {
            if (!in_array($key, $exclude)) {
                $meta->setValue($key, $val);
            }
        }
    }

    public function isPublished($itemId)
    {
        global $wpdb;
        $sql = $wpdb->prepare("SELECT COUNT(*) FROM `{$wpdb->posts}` p JOIN `{$wpdb->postmeta}` m ON p.`ID`=m.`post_id` WHERE p.`post_status`!='trash' AND m.`meta_key`='" . Asg_WpPlugin::PREFIX . "_item_id' AND m.`meta_value`=%s",
            $itemId);
        $result = $wpdb->get_var($sql);

        return !empty($result);
    }

    protected function _htmlEscape($text)
    {
        return esc_attr($text);
    }

    protected function _getTime()
    {
        /**
        * @todo:
        * if (!($schedule = $this->getOption('schedule')) {
        *   return NULL; //all posts immediately
        * }
        */

        if ('immediately' == $this->getOption('publish_type')) {
            return NULL;
        }

        if (!$this->_publication) {
            $start = time() + $this->_publicationDelay;
            $day = floor($start/86400);
            $time = $start % 86400;
            //$step = $this->_scheduleSettings['avg'] > 1
            //    ? $this->_publicationRange / ($this->_scheduleSettings['avg'] - 1)
            //    : 0;

            $schedule = $this->getOption('schedule');

            $todaysPosts = $schedule['avg'];
            $endTime = $this->_publicationStartTime + $this->_publicationRange;
            $weekSchedule = $this->_weekSchedule();
            if (!(($todaysPosts > 1) && (
                ($time>=$this->_publicationStartTime && $time<$endTime)
                || ($endTime>=86400 && $time<($endTime-86400))))
            ) {
                // Nothing to publish today. Move to tomorrow
                $this->_publication = array(
                    'day' => $day+1,
                    'todaysPosts' => $weekSchedule[0],
                    'postCounter' => 0,
                    'dayCounter' => 0,
                    'weekSchedule' => $weekSchedule,
                );
                return NULL;
            }

            // If posting overnight:
            if ($endTime>=86400 && $time<($endTime-86400)) {
                $time += 86400;
                $day--;
            }
            $step = $this->_publicationRange / ($todaysPosts);
            $counter = floor(($time - $this->_publicationStartTime)/$step);

            $this->_publication = array(
                'day' => $day,
                'todaysPosts' => $todaysPosts,
                'postCounter' => $counter,
                'dayCounter' => -1,
                'weekSchedule' => $weekSchedule,
            );

            return NULL;
        }

        $postCounter = $this->_publication['postCounter'];
        if ($postCounter >= $this->_publication['todaysPosts']) {
            while (true) {
                //Move to the next day
                $this->_publication['day']++;
                $dayCounter = ++$this->_publication['dayCounter'];
                if ($dayCounter >= 7) {
                    $dayCounter = $this->_publication['dayCounter'] = 0;
                    $this->_publication['weekSchedule'] = $this->_weekSchedule();
                }
                $todaysPosts = $this->_publication['weekSchedule'][$dayCounter];
                if ($todaysPosts > 0) {
                    $this->_publication['todaysPosts'] = $todaysPosts;
                    $postCounter = $this->_publication['postCounter'] = 0;
                    break;
                }
            }
        }

        $day = $this->_publication['day'];
        $todaysPosts = $this->_publication['todaysPosts'];

        if ($todaysPosts > 1) {
            // Set first post time
            if (0 == $postCounter || !isset($this->_publication['step'])) {
                $this->_publication['step'] = $this->_publicationRange / $todaysPosts;
            }
            $step = $this->_publication['step'];
            $delta = (int)($step * $this->_publicationDeltaRatio);
            $result = $step * ($postCounter + 0.5) + rand(-$delta, $delta);
        } else {
            // 1 post per day: any time whithin publication range
            $result = rand(0, $this->_publicationRange);
        }
        $this->_publication['postCounter']++;

        $result += $this->_publicationStartTime + (86400 * $day);
        if ($result < time()) {
            return NULL;
        }

        return $result;
    }

    protected function _weekSchedule()
    {
        $schedule = $this->getOption('schedule');

        $total = $schedule['weekly'];
        $min = $schedule['min'];
        $max = $schedule['max'];

        $lottery = array();
        for ($i = 0; $i < 7; $i++) {
            $lottery[$i] = array(
                'index' => $i,
                'count' => $min,
            );
        }
        $result = array();

        $count = $total - 7*$min;
        for ($i = 0; $i < $count; $i++) {
            $item = rand(0, count($lottery)-1);
            $lottery[$item]['count']++;
            if ($lottery[$item]['count'] >= $max) {
                $result[$lottery[$item]['index']] = $max;
                unset($lottery[$item]);
                // Reindex array
                $lottery = array_values($lottery);
            }
        }

        foreach ($lottery as $val) {
            $result[$val['index']] = $val['count'];
        }

        return $result;
    }
}