<?php
require_once ASG_APPLICATION_PATH . '/services/Affiliate/Plugin/Scraper.php';

class Asg_Service_Affiliate_Plugin_Scraper_AmazonApi
    extends Asg_Service_Affiliate_Plugin_Scraper
{
    const PREFER_DESCRIPTION_TYPE = 'Product Description';
    const MIN_DESCRIPTION_LENGTH = 500;

    public function scrape(array &$item)
    {
        $this->_scrape($item);

        return $this;
    }

    public function refresh(array &$item)
    {
        $this->_scrape($item, TRUE);

        return $this;
    }

    protected function _scrape(array &$item, $refresh=FALSE)
    {
        $itemData = $this->collectItemData($item['item_id']);

        // Set amazon market if keyword is ASIN
        if (!isset($item['amazon_market'])) {
            $item['amazon_market'] = $this->getOption('amazon_market');
        }

        if (!$this->_isItemAvailable($itemData)) {
            $this->throwError('Item is out of stock. Skipping.');
        }

        // Set amazon market if keyword is ASIN
        if (!isset($item['amazon_market'])) {
            $item['amazon_market'] = $this->getOption('amazon_market');
        }

        if (!$refresh) {
            // title
            if (isset($itemData['ItemAttributes']['Title'])) {
                $item['title'] = (string) $itemData['ItemAttributes']['Title'];
            }

            // text
            $item['text'] = '';
            if (isset($itemData['EditorialReviews']['EditorialReview'])) {
                $content = '';

                if (!isset($itemData['EditorialReviews']['EditorialReview'][0])) {
                    $itemData['EditorialReviews']['EditorialReview'] = array($itemData['EditorialReviews']['EditorialReview']);
                }

                foreach ($itemData['EditorialReviews']['EditorialReview'] as $review) {
                    $current = $review['Content'];
                    //Prefer Product Description
                    if (self::PREFER_DESCRIPTION_TYPE == $review['Source']
                        && strlen($current) >= self::MIN_DESCRIPTION_LENGTH
                    ) {
                        $content = $current;
                        break;
                    }

                    if (strlen($current) > strlen($content)) {
                        $content = $current;
                    }
                }

                $item['text'] = $this->getHelper('Text')
                    ->htmlFragmentToText($content);
            }

            /*
            if ('' == $item['text']) {
                $this->throwError('No product description available');
            }
            */
        }

        $item['amazon_data'] = $itemData;
    }

    public function collectItemData($itemId)
    {
        $params = array(
            'ItemId' => $itemId,
            'IdType' => 'ASIN',
            'ResponseGroup' => implode(',',
                $this->getOption('amazon_response_group')),
        );

        $response = $this->getHelper('AmazonApi')
            ->request('ItemLookup', $params);

        $result = $response->Items->Item;

        if (!$result) {
            if (isset($response->Items->Request->Errors->Error->Message)) {
                $this->throwError("Amazon API returned error: {$response->Items->Request->Errors->Error->Message}");
            }
            $this->throwError('No item data available');
        }

        $result = $this->_xmlToArray($result);

        if (!$this->_isItemAvailable($result)) {
            $this->throwError('Item is out of stock. Skipping.');
        }

        return $result;
    }

    protected function _isItemAvailable(array $item)
    {
        $result = TRUE;
        if (isset($item['OfferSummary'])
            && 0 == $item['OfferSummary']['TotalNew']
            && 0 == $item['OfferSummary']['TotalUsed']
            && 0 == $item['OfferSummary']['TotalCollectible']
            && 0 == $item['OfferSummary']['TotalRefurbished']
            && isset($item['ItemAttributes']['ListPrice'])
        ) {
            $result = FALSE;
            // Check variations
            if (isset($item['Variations']['Item'])) {
                foreach ($item['Variations']['Item'] as $variation) {
                    if (isset($variation['Offers']['Offer'])
                        || isset($variation['Offer'])
                    ) {
                        $result = TRUE;
                        break;
                    }
                }
            }
        }

        return $result;
    }

    protected function _xmlToArray($xml)
    {
        $result = (array) $xml;
        foreach ($result as $key=>$element) {
            if (is_object($element) || is_array($element)) {
                $result[$key] = $this->_xmlToArray($element);
            }
        }

        return $result;
    }
}