<?php
require_once ASG_APPLICATION_PATH . '/services/Affiliate/Plugin/Scraper.php';

class Asg_Service_Affiliate_Plugin_Scraper_AmazonHtml
    extends Asg_Service_Affiliate_Plugin_Scraper
{
    protected $_textHelper;

    protected $_removeLinkTexts;

    protected $_encodingDetectOrder = array(
        'UTF-8',
        'ISO-8859-1',
        'CP1251',
    );

    public function init()
    {
        parent::init();

        $this->_textHelper = $this->getHelper('Text');
    }

    public function refresh(array &$item)
    {
        $this->scrape($item, TRUE);
        $this->scrapeReviews($item);

        return $this;
    }

    public function scrape(array &$item, $refresh=FALSE)
    {
        $url = $item['amazon_data']['DetailPageURL'];

        $html = $this->_getHtml($url, 'product page');

        if (!$refresh) {
            // Extract meta description
            if (preg_match('~<meta\b\s+name="description"\s+content=["\'](.*?)["\']~isu', $html, $matches)) {
                $description = trim($matches[1]);
                if ('' != $description) {
                    $description = preg_replace('~Amazon.com:?~isu', '', $description);

                    $description = $this->_textHelper->htmlDecode($description);
                    $description = trim(preg_replace('~\s+~su', ' ', $description));

                    $item['description'] = $description;
                }
            } else {
                //DebugBreak();
            }

            // Extract meta keywords
            if (preg_match('/<meta\b\s+name="keywords"\s+content=["\'](.*?)["\']/isu', $html, $matches)) {
                $keywords = trim($matches[1]);
                $cfg = Asg_Application::getConfig()->affiliate->options;
                $keywordMaxCount = (int)$cfg->keyword_max_count;
                $keywordMaxLength = (int)$cfg->keyword_max_length;

                if ('' != $keywords) {
                    $keywords = explode(',', $this->_textHelper->htmlDecode($matches[1]));
                    $resultKeywords = array();

                    foreach ($keywords as $keyword) {
                        $keyword = trim(preg_replace('~\s+~su', ' ', $keyword));
                        if ('' != $keyword && strlen($keyword) <= $keywordMaxLength) {
                            $resultKeywords[] = $keyword;
                        }
                        if (count($resultKeywords) >= $keywordMaxCount) {
                            break;
                        }
                    }

                    $item['keywords'] = $resultKeywords;
                }
            } else {
                //DebugBreak();
            }

            // Product Details
            $item['product_details'] = $this->_extractDetails($html, '(?:Product Details|Detalles del producto|Produktinformation)');

            // Technical Details
            $item['technical_details'] = $this->_extractDetails($html, '(?:Technical Details|Descripción técnica|Technische Details)');
        }

        // Star rating
        $item['star_rating'] = $this->_extractStarRating($html);

        return $this;
    }

    public function scrapeReviews(array &$item)
    {
        $url = $this->_getReviewsUrl($item);
        $html = $this->_getHtml($url, 'customer reviews');
        $item['customer_reviews'] = $this->_extractCustomerReviews($html);

        return $this;
    }

    protected function _getHtml($url, $pageName)
    {
        if (!is_null($url)) {
            // Load HTML
            $http = $this->getHelper('Http')->getClient();

            try {
                $response = $http->setUri($url)->request();
            } catch (AsgZend_Http_Client_Exception $e) {
                $this->throwError("Cannot load {$pageName} ({$e->getMessage()})", 'fatal');
            }
            catch (Exception $e) {
                $this->throwError("Unexpected error while loading {$pageName} ({$e->getMessage()})", 'critical');
            }
            if ($response->isError()) {
                $this->throwError("Loading {$pageName} failed due to HTTP error #{$response->getStatus()} ({$response->getMessage()})", 'fatal');
            }

            return $this->_getUtf8Html($response);
        }
        return NULL;
    }

    protected function _getReviewsUrl($item)
    {
        $foundReviewsLink = false;
        if (isset($item['amazon_data']['ItemLinks'])) {
            foreach ($item['amazon_data']['ItemLinks'] as $itemLinks) {
                foreach ($itemLinks as $itemLink) {
                    foreach ($itemLink as $key=>$val) {
                        if ($foundReviewsLink) {
                            return $val;
                        }
                        if ($key == 'Description' && $val == 'All Customer Reviews') {
                            $foundReviewsLink = true;
                        }
                    }
                }
            }
        }

        // Amazon.es does not return ItemLinks
        if (isset($item['amazon_data']['CustomerReviews']['IFrameURL'])
            && isset($item['amazon_data']['CustomerReviews']['HasReviews'])
            && 'true' == $item['amazon_data']['CustomerReviews']['HasReviews']
        ) {
            return $item['amazon_data']['CustomerReviews']['IFrameURL'];
        }

        return NULL;
    }

    protected function _extractDetails($html, $title, $ignore=array())
    {
        $result = array();

        $ignore = $this->getOption('amazon_ignore_details');

        // Parse list
        if (preg_match("~>\s*{$title}\s*<.{0,1000}?<\s*ul\b[^>]*>(.+?)<(?:h2|hr)~isu", $html,
            $matches)
        ) {

            $html = $this->_filterDetailsHtml($matches[1]);

            // Replace level 2 LI tags with LI2
            if (preg_match_all('~<\s*ul\b[^>]*>(.*?<\s*/?\s*li\b[^>]*>.*?)<\s*/\s*ul\s*>~isu', $html, $matches)) {
                foreach ($matches[1] as $match) {
                    $replace = preg_replace('~(<\s*/?\s*)li(\b[^>]*>)~isu', '${1}li2${2}', $match);
                    $html = str_replace($match, $replace, $html);
                }
            }

            // Split list items
            $items = preg_split('~<\s*/?\s*li\b[^>]*>~isu', $html, -1, PREG_SPLIT_NO_EMPTY);

            // Remove the last item as it's not a part of the list
            unset($items[count($items)-1]);

            foreach ($items as $item) {
                $item = trim($item);
                if ('' == $item) {
                    continue;
                }
                if (preg_match('~<b>\s*(.*?)\:?\s*</b>(.*)~isu', $item, $matches)) {
                    $name = $this->_filterDetailsValue($matches[1]);
                    if (in_array($name, $ignore)) {
                        continue;
                    }
                    $value = $matches[2];
                } else {
                    $name = NULL;
                    $value = $item;
                }

                $value = $this->_parseDetailsValue($value);
                if (!$value) {
                    continue;
                }

                // Add to result
                if (!is_null($name)) {
                    $result[$name] = $value;
                } else {
                    $result[] = $value;
                }
            }

        // Parse DIV list
        } else if (preg_match("~>\s*{$title}\s*<.{0,1000}?<div class=\"content\">(.*?)(?:<hr|Rights \& Requirements)~isu", $html,
            $matches)
        ) {
            $html = $this->_filterDetailsHtml($matches[1]);

            // Split list items
            $items = preg_split('~<\s*/?\s*div\b[^>]*>~isu', $html, -1, PREG_SPLIT_NO_EMPTY);

            // Remove the last item as it's not a part of the list
            unset($items[count($items)-1]);

            foreach ($items as $item) {
                $item = trim($item);
                if ('' == $item) {
                    continue;
                }
                if (preg_match('~<\s*span\b[^>]*>\s*(.*?)\:?\s*<\s*/\s*span\s*>(.*)~isu', $item, $matches)) {
                    $name = $this->_filterDetailsValue($matches[1]);
                    if (in_array($name, $ignore)) {
                        continue;
                    }
                    $value = $matches[2];
                } else {
                    $name = NULL;
                    $value = $item;
                }

                $value = $this->_parseDetailsValue($value);

                // Add to result
                if (!is_null($name)) {
                    $result[$name] = $value;
                } else {
                    $result[] = $value;
                }
            }

        // Parse table
        } else if (preg_match("~>\s*{$title}\s*<.{0,1000}?<\s*table\b[^>]*>(.*?)<\s*/table\s*>~isu", $html,
            $matches)
        ) {
            $html = $matches[1];

            // Possibly table in table
            if (preg_match("~<\s*table\b[^>]*>(.*)~isu", $html, $matches)) {
                $html = $matches[1];
            }

            $html = $this->_filterDetailsHtml($html);

            // Parse table
            if (preg_match_all('~<\s*tr[^>]*>.*?<\s*td[^>]*>(.*?)<\s*/td\s*>\s*<\s*td[^>]*>(.*?)<\s*/td\s*>~isu', $html,
                $matches, PREG_SET_ORDER)
            ) {
                foreach ($matches as $match) {
                    $name = trim($this->_textHelper->htmlDecode(
                        strip_tags($match[1])));
                    if ('' == $name) {
                        continue;
                    }
                    $result[$name] = $this->_parseDetailsValue($match[2]);
                }
            }
        }

        return $result? $result : NULL;
    }

    protected function _parseDetailsValue($value)
    {
        // Try extract sublist
        $valueList = preg_split('~<\s*/?\s*li2?\b[^>]*>~isu', $value, -1, PREG_SPLIT_NO_EMPTY);
        if (count($valueList) > 1) {
            unset($valueList[count($valueList)-1]);
        } else if (FALSE !== strpos($value, '&bull;')) {
            $valueList = explode('&bull;', $value);
        }

        $valueItems = array();
        if (count($valueList) > 1) {
            foreach ($valueList as $valueItem) {
                $valueItem = $this->_filterDetailsValue($valueItem);
                if ('' == $valueItem) {
                    continue;
                }

                $valueItems[] = $valueItem;
            }
        }

        return $valueItems? $valueItems : $this->_filterDetailsValue($value);
    }

    protected function _filterDetailsHtml($html)
    {
        // Remove scripts & styles
        $html = preg_replace('~<\s*script\b[^>]*>.*?<\s*/script\s*>~isu', '', $html);
        $html = preg_replace('~<\s*style\b[^>]*>.*?<\s*/style\s*>~isu', '', $html);

        //Remove hidden text:
        $html = preg_replace('~<\s*([\w]+)[^>]+\bdisplay\s*\:\s*none\b[^>]*>.*?<\s*/\s*\\1\s*>~isu', '', $html);
        $html = preg_replace('~<\s*([\w]+)[^>]+\bvisibility\s*\:\s*hidden\b[^>]*>.*?<\s*/\s*\\1\s*>~isu', '', $html);

        // Remove hyperlinks in brackets
        $html = preg_replace('~\(\s*<\s*a\b[^>]*>.*?<\s*/a\s*>\s*\)~isu', '', $html);

        // Remove images
        $html = preg_replace('~<\s*img\b[^>]*>~isu', '', $html);

        // Remove special links and other elements
        $texts = $this->getOption('amazon_remove_detail_elements');
        if ($texts) {
            foreach ($texts as $i=>$text) {
                $texts[$i] = preg_quote($text, '~');
            }
            $texts = implode('|', $texts);
            $html = preg_replace("~<\s*(\w+)\b[^>]*>\s*(?:{$texts})\s*<\s*/\\1\s*>~isu", '', $html);
        }

        return $html;
    }

    protected function _filterDetailsValue($text)
    {
        $text = str_replace("\n", '', $text);
        $text = strip_tags($text);
        $text = $this->_textHelper->htmlDecode($text);
        $text = preg_replace('~  +~u', ' ', $text);
        $text = trim($text);

        return $text;
    }

    protected function _filterReviewsValue($text)
    {
        $text = str_replace("\n", '', $text);
        $text = htmlspecialchars_decode($text); // used instead of htmlDecode due to unresolved encoding issues
        $text = preg_replace('~  +~u', ' ', $text);
        $text = trim($text);
        $text = wp_check_invalid_utf8($text, TRUE);

        return $text;
    }

    protected function _extractStarRating($html)
    {
        $result = NULL;

        // Cut featured items
        $html = preg_replace('~<div[^>]+\bid="[^"]+-feature".*$~ismu', '', $html);

        if (preg_match('~\btitle="(\d(?:\.\d)?) (?:out of 5 stars|von 5 Sternen|de un máximo de 5 estrellas)"~isu', $html,
            $matches)
        ) {
            $result = (double)$matches[1];
        }

        return $result;
    }

    protected function _extractCustomerReviews($html)
    {
        $result = array();

        if (preg_match('~<table\s+id="productReviews".*?<td width="90%">(.*?)</td>~isu', $html, $matchesTable)) {
            $reviewBlocks = explode('<!-- BOUNDARY -->', $matchesTable[1]);
            foreach ($reviewBlocks as $reviewBlock) {
               if (preg_match('~<div style="margin-left:0.5em;">(.*)</div>~isu', $reviewBlock, $reviewItem)){
                   $data = array();
                   if (preg_match('~<span style="vertical-align:middle;"><b>(.*?)</b>~isu', $reviewItem[1], $matches)) {
                       $data['title'] = $this->_filterReviewsValue($matches[1]);
                   }
                   if (preg_match('~<div style="margin-bottom:0.5em;">(.*?helpful):.*?</div>~isu', $reviewItem[1], $matches)) {
                       $data['helpful'] = $this->_filterReviewsValue($matches[1]);
                   }
                   if (preg_match('~<span class="swSprite.*?><span>([0-9\.]+)~isu', $reviewItem[1], $matches)) {
                       $data['star'] = $this->_filterReviewsValue($matches[1]);
                   }
                   if (preg_match('~</b>, <nobr>(.*?)</nobr>~isu', $reviewItem[1], $matches)) {
                       $data['date'] = $this->_filterReviewsValue($matches[1]);
                   }
                   if (preg_match('~>By&nbsp;</div>.*?<a href="(.*?)".*?<span .*?>(.*?)</span></a>(.*?)-~isu', $reviewItem[1], $matches)) {
                       $data['author']['url'] = $this->_filterReviewsValue($matches[1]);
                       $data['author']['name'] = $this->_filterReviewsValue($matches[2]);
                       $data['author']['address'] = $this->_filterReviewsValue(str_replace(array('(',')'), '', $matches[3]));
                   }
                   if (preg_match('~<div class="tiny".*?</div>(.*?)<div style="padding-top:~isu', $reviewItem[1], $matches)) {
                       $text = $matches[1];

                       // Additional filters
                       $text = preg_replace('~<\s*div[^>]*>.*?<\s*/\s*div\s*>~isu', '', $text);

                       $data['text'] = $this->_filterReviewsValue($text);
                   }
                   if (!is_null($data['text'])) {
                       $result[] = $data;
                   }
               }
            }
        }

        return $result? $result : NULL;
    }

    /**
    * Continues regex replace until pattern not found in subject
    *
    * @param string $pattern
    * @param string $subject
    */
    protected function _multiPregReplace($pattern, $replacement, $subject)
    {
        do {
            $subject = preg_replace($pattern, $replacement, $subject, -1, $count);
        } while ($count > 0);

        return $subject;
    }

    public function _getUtf8Html(AsgZend_Http_Response $response)
    {
        $result = $response->getBody();
        $encoding = NULL;

        if (preg_match('~<meta\b.*?\bhttp-equiv="Content-Type".*?\bcontent="(.*?)"~is', $result, $matches)) {
            $encoding = $this->_extractEncoding($matches[1]);
        }

        if ('' == $encoding) {
            $contentType = strtolower($response->getHeader('Content-Type'));
            if ('' != $contentType && strpos($contentType, 'text/html')===0) {
                $encoding = $this->_extractEncoding($contentType);
            }
        }

        if ('' == $encoding) {
            $encoding = function_exists('mb_detect_encoding')
                ? strtolower(mb_detect_encoding($result, $this->_encodingDetectOrder))
                : null;
        }

        if (!in_array($encoding, array('utf-8', 'utf8'))) {
            if ('' != $encoding) {
                $result = iconv($encoding, 'utf-8', $result);
            }

            if ('' == trim($result)) {
                $result = $response->getBody();
            }
        }

        return wp_check_invalid_utf8($result, TRUE);
    }

    protected function _extractEncoding($contentType)
    {
        if (preg_match('/\bcharset=(.*)(;|$)/is', $contentType, $matches)) {
            $result = strtolower(trim($matches[1]));
        }
        else {
            $result = '';
        }

        return $result;
    }
}